# -*- coding: utf-8 -*-
"""
@author: yq
@time: 2024/11/1
@desc: 
"""
from typing import Dict

import pandas as pd
import scorecardpy as sc
from matplotlib import pyplot as plt
from sklearn.linear_model import LogisticRegression

from entitys import TrainConfigEntity, DataPreparedEntity, MetricFucEntity
from init import f_get_save_path
from .model_base import ModelBase


class ModelLr(ModelBase):
    def __init__(self, train_config: TrainConfigEntity):
        super().__init__(train_config)
        self.lr = LogisticRegression(penalty='l1', C=0.9, solver='saga', n_jobs=-1)

    def train(self, data: DataPreparedEntity, *args, **kwargs) -> Dict[str, MetricFucEntity]:
        train_data = data.train_data
        train_y = train_data.get_Ydata()
        test_data = data.test_data
        test_y = test_data.get_Ydata()
        self.lr.fit(train_data.get_Xdata(), train_y)

        train_prob = self.lr.predict_proba(train_data.get_Xdata())[:, 1]
        test_prob = self.lr.predict_proba(test_data.get_Xdata())[:, 1]

        image_path_list = []
        train_perf = sc.perf_eva(train_y, train_prob, title="train", show_plot=True)
        path = f_get_save_path(f"train_perf.png")
        train_perf["pic"].savefig(path)
        image_path_list.append(path)

        test_perf = sc.perf_eva(test_y, test_prob, title="test", show_plot=True)
        path = f_get_save_path(f"test_perf.png")
        test_perf["pic"].savefig(path)
        image_path_list.append(path)

        train_auc = train_perf["KS"]
        train_ks = train_perf["AUC"]

        test_auc = test_perf["KS"]
        test_ks = test_perf["AUC"]

        metric_value_dict = {}
        df = pd.DataFrame()
        df["样本集"] = ["训练集", "测试集"]
        df["AUC"] = [train_auc, test_auc]
        df["KS"] = [train_ks, test_ks]

        metric_value_dict["模型结果"] = MetricFucEntity(table=df, image_path=image_path_list, image_size=5)

        return metric_value_dict

    def predict_prob(self, x: pd.DataFrame, *args, **kwargs):
        return self.lr.predict_proba(x)[:, 1]

    def predict(self, x: pd.DataFrame, *args, **kwargs):
        return self.lr.predict(x)

    def export_model_file(self):
        pass


if __name__ == "__main__":
    pass
